/*
**    Projet etudiants CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef OPENGL_DRAWER_HPP
#define OPENGL_DRAWER_HPP

#include <mesh.hpp>
#include <v3.hpp>


namespace cpe
{

    /** \brief Helper class for OpenGL calls */
    class opengl_drawer
    {
    public:

        /** \brief draw mesh with given normals

          \param mesh m: the mesh
          \param std::vector<double> normal: the normal of the meshes
          Normal size must be either vertex_number():smooth shading or triangle_number():flat shading
      */
        static void draw(const mesh& m,const std::vector<double>& normal);
        /** \brief draw mesh (without normal information: only draw edges) */
        static void draw(const mesh& m);


        /** \brief draw normal associated to a given mesh
          \param mesh m: the mesh
          \param std::vector<double> normal: the per_face or per_vertex normal
          \param L_scaling: scale the normal size
      */
        static void draw_normals(const mesh& m,const std::vector<double>& normal,const double& L_scaling=1.0);


        /** \brief generate a texture identifiant from a data vector */
        static void generate_texture(const unsigned int& id_texture,const std::string& filename);



    private:

        /** \brief draw mesh edges only */
        static void draw_edges(const mesh& m);
        /** \brief draw mesh with polygon per triangle */
        static void draw_per_polygon_normal(const mesh& m,const std::vector<double>& normal);
        /** \brief draw mesh with polygon per vertex */
        static void draw_per_vertex_normal(const mesh& m,const std::vector<double>& normal);

    };
}

#endif
