/*
**    Projet etudiants CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <texture.hpp>

#include <fstream>
#include <string_helper.hpp>

namespace cpe
{
    texture::texture()
        :N_x(0),N_y(0)
    {}

    void texture::load_picture(const std::string& filename)
    {
        std::string extension=filename.substr(filename.rfind('.'),filename.size());

        if(extension==".ppm")
            load_ppm_picture(filename);
        else
        {
            std::cout<<"texture::load_picture unkown format: "<<extension<<std::endl;
            exit(-1);
        }
    }

    void texture::load_ppm_picture(const std::string& filename)
    {
        picture.clear();

        std::ifstream stream(filename.c_str());
        if(!stream.good())
            throw std::string("texture::load_ppm_picture("+filename+") cannot be opened");

        //go to magic number
        std::string buffer;
        buffer="#";
        while(buffer[0]=='#')
            std::getline(stream,buffer);
        if(buffer!="P3")
            throw std::string("texture::load_ppm_picture("+filename+") did not find P3 magic number: "+buffer);

        //Nx
        buffer="#";
        while(buffer[0]=='#')
            std::getline(stream,buffer);
        std::vector<std::string> size=cpe::string_converter::delete_empty(cpe::string_tokenizer::tokenize(buffer));
        if(size.size()<2)
            throw std::string("texture::load_ppm_picture("+filename+") cannot read size: "+buffer);
        N_x=cpe::string_converter::value_of<unsigned int>(size[0]);
        N_y=cpe::string_converter::value_of<unsigned int>(size[1]);

        //color number
        stream>>buffer;



        for(unsigned int k_x=0;k_x<N_x;k_x++)
        {
            for(unsigned int k_y=0;k_y<N_y;k_y++)
            {
                for(unsigned int k_dim=0;k_dim<3;k_dim++)
                {
                    stream>>buffer;
                    unsigned int val=cpe::string_converter::value_of<unsigned int>(buffer);
                    picture.push_back(val);
                }
                picture.push_back(255);
            }
        }


        //only store RGBA values
        depth=4;

        stream.close();

    }

    void texture::export_ppm(const std::string& filename) const
    {
        std::ofstream stream(filename.c_str(),std::ofstream::out);
        if(!stream.good())
            throw std::string("Error in Texture::export_ppm("+filename+"...) : cannot open file \n");

        //magic number
        stream<<"P3 \n";
        //size
        stream<<N_x<<" "<<N_y<<" \n";
        //color number
        stream<<"255"<<std::endl;

        for(unsigned int k2=0;k2<N_x;++k2)
        {
            for(unsigned int k1=0;k1<N_y;++k1)
            {
                stream<<static_cast<unsigned int>(picture[4*(N_y*k2+k1)+0])<<" "<<
                        static_cast<unsigned int>(picture[4*(N_y*k2+k1)+1])<<" "<<
                        static_cast<unsigned int>(picture[4*(N_y*k2+k1)+2])<<std::endl;
            }
        }
        stream.close();
    }

    const unsigned int& texture::get_Nx() const
    {
        return N_x;
    }
    const unsigned int& texture::get_Ny() const
    {
        return N_y;
    }
    const std::vector<unsigned char>& texture::get_data() const
    {
        return picture;
    }
    const unsigned int& texture::get_depth() const
    {
        return depth;
    }
}
